@@ -12,10 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import annotations
-from typing import TYPE_CHECKING
+
+from typing import TYPE_CHECKING, Any
 
 if TYPE_CHECKING:
     from supertokens_python.recipe.emailpassword.interfaces import APIOptions, APIInterface
+
+from supertokens_python.exceptions import raise_bad_input_exception
+
 from .utils import validate_form_fields_or_throw_error
 
 
@@ -23,10 +27,12 @@ async def handle_sign_up_api(api_implementation: APIInterface, api_options: APIO
     if api_implementation.disable_sign_up_post:
         return None
     body = await api_options.request.json()
-    form_fields_raw = body['formFields'] if 'formFields' in body else []
+    if body is None:
+        raise_bad_input_exception('Please provide a JSON body')
+    form_fields_raw: Any = body['formFields'] if 'formFields' in body else []
     form_fields = await validate_form_fields_or_throw_error(api_options.config.sign_up_feature.form_fields,
                                                             form_fields_raw)
-    response = await api_implementation.sign_up_post(form_fields, api_options)
+    response = await api_implementation.sign_up_post(form_fields, api_options, {})
 
     api_options.response.set_json_content(response.to_json())
 
