@@ -12,15 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from abc import ABC, abstractmethod
-from typing import Union, List
-from .types import User, DeviceType
-try:
-    from typing import Literal
-except ImportError:
-    from typing_extensions import Literal
-from .utils import PasswordlessConfig
+from typing import Any, Dict, List, Union
+
 from supertokens_python.framework import BaseRequest, BaseResponse
-from supertokens_python.recipe.session import Session
+from supertokens_python.recipe.session import SessionContainer
+from typing_extensions import Literal
+
+from .types import DeviceType, User
+from .utils import PasswordlessConfig
 
 
 class CreateCodeResult(ABC):
@@ -99,13 +98,15 @@ class CreateNewCodeForDeviceOkResult(CreateNewCodeForDeviceResult):
         self.is_ok = True
 
 
-class CreateNewCodeForDeviceRestartFlowErrorResult(CreateNewCodeForDeviceResult):
+class CreateNewCodeForDeviceRestartFlowErrorResult(
+        CreateNewCodeForDeviceResult):
     def __init__(self):
         super().__init__('RESTART_FLOW_ERROR')
         self.is_restart_flow_error = True
 
 
-class CreateNewCodeForDeviceUserInputCodeAlreadyUsedErrorResult(CreateNewCodeForDeviceResult):
+class CreateNewCodeForDeviceUserInputCodeAlreadyUsedErrorResult(
+        CreateNewCodeForDeviceResult):
     def __init__(self):
         super().__init__('USER_INPUT_CODE_ALREADY_USED_ERROR')
         self.is_user_input_code_already_used_error = True
@@ -122,15 +123,18 @@ class ConsumeCodeResult(ABC):
                  failed_code_input_attempt_count: Union[int, None] = None,
                  maximum_code_input_attempts: Union[int, None] = None
                  ):
-        self.status = status
-        self.created_new_user = created_new_user
-        self.user = user
-        self.failed_code_input_attempt_count = failed_code_input_attempt_count
-        self.maximum_code_input_attempts = maximum_code_input_attempts
-        self.is_ok = False
-        self.is_incorrect_user_input_code_error = False
-        self.is_expired_user_input_code_error = False
-        self.is_restart_flow_error = False
+        self.status: Literal['OK',
+                             'INCORRECT_USER_INPUT_CODE_ERROR',
+                             'EXPIRED_USER_INPUT_CODE_ERROR',
+                             'RESTART_FLOW_ERROR'] = status
+        self.created_new_user: Union[bool, None] = created_new_user
+        self.user: Union[User, None] = user
+        self.failed_code_input_attempt_count: Union[int, None] = failed_code_input_attempt_count
+        self.maximum_code_input_attempts: Union[int, None] = maximum_code_input_attempts
+        self.is_ok: bool = False
+        self.is_incorrect_user_input_code_error: bool = False
+        self.is_expired_user_input_code_error: bool = False
+        self.is_restart_flow_error: bool = False
 
 
 class ConsumeCodeOkResult(ConsumeCodeResult):
@@ -140,7 +144,8 @@ class ConsumeCodeOkResult(ConsumeCodeResult):
 
 
 class ConsumeCodeIncorrectUserInputCodeErrorResult(ConsumeCodeResult):
-    def __init__(self, failed_code_input_attempt_count: int, maximum_code_input_attempts: int):
+    def __init__(self, failed_code_input_attempt_count: int,
+                 maximum_code_input_attempts: int):
         super().__init__('INCORRECT_USER_INPUT_CODE_ERROR',
                          failed_code_input_attempt_count=failed_code_input_attempt_count,
                          maximum_code_input_attempts=maximum_code_input_attempts)
@@ -148,7 +153,8 @@ class ConsumeCodeIncorrectUserInputCodeErrorResult(ConsumeCodeResult):
 
 
 class ConsumeCodeExpiredUserInputCodeErrorResult(ConsumeCodeResult):
-    def __init__(self, failed_code_input_attempt_count: int, maximum_code_input_attempts: int):
+    def __init__(self, failed_code_input_attempt_count: int,
+                 maximum_code_input_attempts: int):
         super().__init__('EXPIRED_USER_INPUT_CODE_ERROR',
                          failed_code_input_attempt_count=failed_code_input_attempt_count,
                          maximum_code_input_attempts=maximum_code_input_attempts)
@@ -162,7 +168,8 @@ class ConsumeCodeRestartFlowErrorResult(ConsumeCodeResult):
 
 
 class UpdateUserResult(ABC):
-    def __init__(self, status: Literal['OK', 'UNKNOWN_USER_ID_ERROR', 'EMAIL_ALREADY_EXISTS_ERROR', 'PHONE_NUMBER_ALREADY_EXISTS_ERROR']):
+    def __init__(self, status: Literal['OK', 'UNKNOWN_USER_ID_ERROR',
+                 'EMAIL_ALREADY_EXISTS_ERROR', 'PHONE_NUMBER_ALREADY_EXISTS_ERROR']):
         self.status = status
 
 
@@ -212,68 +219,74 @@ class RecipeInterface(ABC):
 
     @abstractmethod
     async def create_code(self,
-                          email: Union[None, str] = None,
-                          phone_number: Union[None, str] = None,
-                          user_input_code: Union[None, str] = None) -> CreateCodeResult:
+                          email: Union[None, str],
+                          phone_number: Union[None, str],
+                          user_input_code: Union[None, str],
+                          user_context: Dict[str, Any]) -> CreateCodeResult:
         pass
 
     @abstractmethod
     async def create_new_code_for_device(self,
                                          device_id: str,
-                                         user_input_code: Union[str, None] = None) -> CreateNewCodeForDeviceResult:
+                                         user_input_code: Union[str, None],
+                                         user_context: Dict[str, Any]) -> CreateNewCodeForDeviceResult:
         pass
 
     @abstractmethod
     async def consume_code(self,
                            pre_auth_session_id: str,
-                           user_input_code: Union[str, None] = None,
-                           device_id: Union[str, None] = None,
-                           link_code: Union[str, None] = None) -> ConsumeCodeResult:
+                           user_input_code: Union[str, None],
+                           device_id: Union[str, None],
+                           link_code: Union[str, None],
+                           user_context: Dict[str, Any]) -> ConsumeCodeResult:
         pass
 
     @abstractmethod
-    async def get_user_by_id(self, user_id: str) -> Union[User, None]:
+    async def get_user_by_id(self, user_id: str, user_context: Dict[str, Any]) -> Union[User, None]:
         pass
 
     @abstractmethod
-    async def get_user_by_email(self, email: str) -> Union[User, None]:
+    async def get_user_by_email(self, email: str, user_context: Dict[str, Any]) -> Union[User, None]:
         pass
 
     @abstractmethod
-    async def get_user_by_phone_number(self, phone_number: str) -> Union[User, None]:
+    async def get_user_by_phone_number(self, phone_number: str, user_context: Dict[str, Any]) -> Union[User, None]:
         pass
 
     @abstractmethod
-    async def update_user(self, user_id: str, email: Union[str, None] = None, phone_number: Union[str, None] = None) -> UpdateUserResult:
+    async def update_user(self, user_id: str,
+                          email: Union[str, None], phone_number: Union[str, None], user_context: Dict[str, Any]) -> UpdateUserResult:
         pass
 
     @abstractmethod
-    async def revoke_all_codes(self, email: Union[str, None] = None, phone_number: Union[str, None] = None) -> RevokeAllCodesResult:
+    async def revoke_all_codes(self,
+                               email: Union[str, None], phone_number: Union[str, None], user_context: Dict[str, Any]) -> RevokeAllCodesResult:
         pass
 
     @abstractmethod
-    async def revoke_code(self, code_id: str) -> RevokeCodeResult:
+    async def revoke_code(self, code_id: str, user_context: Dict[str, Any]) -> RevokeCodeResult:
         pass
 
     @abstractmethod
-    async def list_codes_by_email(self, email: str) -> List[DeviceType]:
+    async def list_codes_by_email(self, email: str, user_context: Dict[str, Any]) -> List[DeviceType]:
         pass
 
     @abstractmethod
-    async def list_codes_by_phone_number(self, phone_number: str) -> List[DeviceType]:
+    async def list_codes_by_phone_number(self, phone_number: str, user_context: Dict[str, Any]) -> List[DeviceType]:
         pass
 
     @abstractmethod
-    async def list_codes_by_device_id(self, device_id: str) -> Union[DeviceType, None]:
+    async def list_codes_by_device_id(self, device_id: str, user_context: Dict[str, Any]) -> Union[DeviceType, None]:
         pass
 
     @abstractmethod
-    async def list_codes_by_pre_auth_session_id(self, pre_auth_session_id: str) -> Union[DeviceType, None]:
+    async def list_codes_by_pre_auth_session_id(self, pre_auth_session_id: str,
+                                                user_context: Dict[str, Any]) -> Union[DeviceType, None]:
         pass
 
 
 class APIOptions:
-    def __init__(self, request: BaseRequest, response: Union[BaseResponse, None], recipe_id: str,
+    def __init__(self, request: BaseRequest, response: BaseResponse, recipe_id: str,
                  config: PasswordlessConfig, recipe_implementation: RecipeInterface):
         self.request = request
         self.response = response
@@ -288,7 +301,8 @@ class CreateCodePostResponse(ABC):
         status: Literal['OK', 'GENERAL_ERROR'],
         device_id: Union[str, None] = None,
         pre_auth_session_id: Union[str, None] = None,
-        flow_type: Literal['USER_INPUT_CODE', 'MAGIC_LINK', 'USER_INPUT_CODE_AND_MAGIC_LINK'] = None,
+        flow_type: Union[None, Literal['USER_INPUT_CODE', 'MAGIC_LINK',
+                                       'USER_INPUT_CODE_AND_MAGIC_LINK']] = None,
         message: Union[str, None] = None
     ):
         self.status = status
@@ -300,7 +314,7 @@ class CreateCodePostResponse(ABC):
         self.is_general_error = False
 
     @abstractmethod
-    def to_json(self):
+    def to_json(self) -> Dict[str, Any]:
         pass
 
 
@@ -357,7 +371,7 @@ class ResendCodePostResponse(ABC):
         self.is_restart_flow_error = False
 
     @abstractmethod
-    def to_json(self):
+    def to_json(self) -> Dict[str, Any]:
         pass
 
 
@@ -409,35 +423,47 @@ class ConsumeCodePostResponse(ABC):
         ],
         created_new_user: Union[bool, None] = None,
         user: Union[User, None] = None,
-        session: Union[Session, None] = None,
+        session: Union[SessionContainer, None] = None,
         message: Union[str, None] = None,
         failed_code_input_attempt_count: Union[int, None] = None,
         maximum_code_input_attempts: Union[int, None] = None
     ):
-        self.status = status
-        self.session = session
-        self.created_new_user = created_new_user
-        self.user = user
-        self.failed_code_input_attempt_count = failed_code_input_attempt_count
-        self.maximum_code_input_attempts = maximum_code_input_attempts
-        self.message = message
-        self.is_ok = False
-        self.is_general_error = False
-        self.is_restart_flow_error = False
-        self.is_incorrect_user_input_code_error = False
-        self.is_expired_user_input_code_error = False
+        self.status: Literal[
+            'OK',
+            'GENERAL_ERROR',
+            'RESTART_FLOW_ERROR',
+            'INCORRECT_USER_INPUT_CODE_ERROR',
+            'EXPIRED_USER_INPUT_CODE_ERROR'
+        ] = status
+        self.session: Union[SessionContainer, None] = session
+        self.created_new_user: Union[bool, None] = created_new_user
+        self.user: Union[User, None] = user
+        self.failed_code_input_attempt_count: Union[int, None] = failed_code_input_attempt_count
+        self.maximum_code_input_attempts: Union[int, None] = maximum_code_input_attempts
+        self.message: Union[str, None] = message
+        self.is_ok: bool = False
+        self.is_general_error: bool = False
+        self.is_restart_flow_error: bool = False
+        self.is_incorrect_user_input_code_error: bool = False
+        self.is_expired_user_input_code_error: bool = False
 
     @abstractmethod
-    def to_json(self):
+    def to_json(self) -> Dict[str, Any]:
         pass
 
 
 class ConsumeCodePostOkResponse(ConsumeCodePostResponse):
-    def __init__(self, created_new_user: bool, user: User, session: Session):
-        super().__init__(status='OK', created_new_user=created_new_user, user=user, session=session)
+    def __init__(self, created_new_user: bool, user: User, session: SessionContainer):
+        super().__init__(
+            status='OK',
+            created_new_user=created_new_user,
+            user=user,
+            session=session)
         self.is_ok = True
 
     def to_json(self):
+        if self.user is None:
+            raise Exception("Should never come here")
         user = {
             'id': self.user.user_id,
             'time_joined': self.user.time_joined
@@ -489,7 +515,8 @@ class ConsumeCodePostGeneralErrorResponse(ConsumeCodePostResponse):
         }
 
 
-class ConsumeCodePostIncorrectUserInputCodeErrorResponse(ConsumeCodePostResponse):
+class ConsumeCodePostIncorrectUserInputCodeErrorResponse(
+        ConsumeCodePostResponse):
     def __init__(
             self,
             failed_code_input_attempt_count: int,
@@ -509,7 +536,8 @@ class ConsumeCodePostIncorrectUserInputCodeErrorResponse(ConsumeCodePostResponse
         }
 
 
-class ConsumeCodePostExpiredUserInputCodeErrorResponse(ConsumeCodePostResponse):
+class ConsumeCodePostExpiredUserInputCodeErrorResponse(
+        ConsumeCodePostResponse):
     def __init__(
             self,
             failed_code_input_attempt_count: int,
@@ -583,14 +611,16 @@ class APIInterface:
     async def create_code_post(self,
                                email: Union[str, None],
                                phone_number: Union[str, None],
-                               api_options: APIOptions) -> CreateCodePostResponse:
+                               api_options: APIOptions,
+                               user_context: Dict[str, Any]) -> CreateCodePostResponse:
         pass
 
     @abstractmethod
     async def resend_code_post(self,
                                device_id: str,
                                pre_auth_session_id: str,
-                               api_options: APIOptions) -> ResendCodePostResponse:
+                               api_options: APIOptions,
+                               user_context: Dict[str, Any]) -> ResendCodePostResponse:
         pass
 
     @abstractmethod
@@ -599,17 +629,20 @@ class APIInterface:
                                 user_input_code: Union[str, None],
                                 device_id: Union[str, None],
                                 link_code: Union[str, None],
-                                api_options: APIOptions) -> ConsumeCodePostResponse:
+                                api_options: APIOptions,
+                                user_context: Dict[str, Any]) -> ConsumeCodePostResponse:
         pass
 
     @abstractmethod
     async def email_exists_get(self,
                                email: str,
-                               api_options: APIOptions) -> EmailExistsGetResponse:
+                               api_options: APIOptions,
+                               user_context: Dict[str, Any]) -> EmailExistsGetResponse:
         pass
 
     @abstractmethod
     async def phone_number_exists_get(self,
                                       phone_number: str,
-                                      api_options: APIOptions) -> PhoneNumberExistsGetResponse:
+                                      api_options: APIOptions,
+                                      user_context: Dict[str, Any]) -> PhoneNumberExistsGetResponse:
         pass
